require 'yaml'
require 'net/http'
require 'simple_firebase/response'
require 'simple_firebase/exceptions/firebase_error'

module SimpleFirebase
  class Client
    DEFAULT_PAGE_SIZE = 1000
    attr_reader :uri
    attr_reader :key

    def initialize(uri, key)
      @uri = uri
      @key = key
    end

    def fetch_single(resource, key, value)
      raw = fetch(resource, key, value, 1).values.first
      if raw
        if raw[key] == value
          raw
        else
          nil
        end
      else
        nil
      end
    end

    def fetch(resource, key, value, page_size)
      uri = construct_uri(resource, key, value, page_size)

      r = SimpleFirebase::Response.new(Net::HTTP.get_response(uri))

      raise SimpleFirebase::Exceptions::FirebaseError r.body unless r.success?

      r.json_body
    end

    private

    def construct_uri(resource, order_by, start_at, page_size)
      q = { auth: @key, limitToFirst: page_size || DEFAULT_PAGE_SIZE }
      q[:orderBy] = '"' + order_by + '"' if order_by
      q[:startAt] = '"' + start_at.to_s + '"'

      uri = URI("#{@uri}#{resource}.json")
      uri.query = URI.encode_www_form q

      uri
    end
  end
end
